pub struct AttributeChain { /* private fields */ }Expand description
A parsed Rust attribute expression made from a path root and zero or more dot calls.
Accepted roots are:
- a path, such as
RangeValidation::<_>.
Implementations
Sourceimpl AttributeChain
impl AttributeChain
Sourcepub fn parse_with_options(
input: ParseStream<'_>,
options: &ChainParseOptions,
) -> Result<Self>
pub fn parse_with_options( input: ParseStream<'_>, options: &ChainParseOptions, ) -> Result<Self>
Parse an AttributeChain from a ParseStream with custom options.
Errors
Returns syn::Error when the token stream is not a supported path or
dot-call chain, completion probes are disabled for probe syntax, or the
configured completion marker cannot be emitted as a Rust identifier.
Sourcepub fn parse_tokens_with_options(
tokens: TokenStream,
options: &ChainParseOptions,
) -> Result<Self>
pub fn parse_tokens_with_options( tokens: TokenStream, options: &ChainParseOptions, ) -> Result<Self>
Parse an AttributeChain from tokens with custom options.
Errors
Returns syn::Error for the same syntax and option failures as
Self::parse_with_options.
pub fn root(&self) -> &Path
Sourcepub fn root_path(&self) -> &Path
pub fn root_path(&self) -> &Path
Examples found in repository?
examples/derive_field_attrs.rs (line 68)
55fn expand_field_attrs(field: &Field) -> syn::Result<Vec<TokenStream>> {
56 let field_ident = field
57 .ident
58 .as_ref()
59 .expect("named_fields only returns named struct fields");
60 let mut expansions = Vec::new();
61
62 for attr in &field.attrs {
63 if !attr.path().is_ident("attribute_dsl") {
64 continue;
65 }
66
67 let chain = attr.parse_args::<AttributeChain>()?;
68 let root = substitute_infer_in_path(chain.root_path(), &field.ty);
69
70 let calls = chain.calls().iter().map(|call| {
71 let method = call.method();
72 let turbofish = call.turbofish();
73 let args = call.args();
74 quote! { .#method #turbofish (#(#args),*) }
75 });
76
77 let completion = chain
78 .completion_marker()
79 .map(|marker| quote! { .#marker })
80 .unwrap_or_default();
81
82 expansions.push(quote! {
83 let _ = #root::builder_for(stringify!(#field_ident)) #(#calls)* #completion;
84 });
85 }
86
87 Ok(expansions)
88}Sourcepub fn calls(&self) -> &[ChainCall]
pub fn calls(&self) -> &[ChainCall]
Examples found in repository?
examples/derive_field_attrs.rs (line 70)
55fn expand_field_attrs(field: &Field) -> syn::Result<Vec<TokenStream>> {
56 let field_ident = field
57 .ident
58 .as_ref()
59 .expect("named_fields only returns named struct fields");
60 let mut expansions = Vec::new();
61
62 for attr in &field.attrs {
63 if !attr.path().is_ident("attribute_dsl") {
64 continue;
65 }
66
67 let chain = attr.parse_args::<AttributeChain>()?;
68 let root = substitute_infer_in_path(chain.root_path(), &field.ty);
69
70 let calls = chain.calls().iter().map(|call| {
71 let method = call.method();
72 let turbofish = call.turbofish();
73 let args = call.args();
74 quote! { .#method #turbofish (#(#args),*) }
75 });
76
77 let completion = chain
78 .completion_marker()
79 .map(|marker| quote! { .#marker })
80 .unwrap_or_default();
81
82 expansions.push(quote! {
83 let _ = #root::builder_for(stringify!(#field_ident)) #(#calls)* #completion;
84 });
85 }
86
87 Ok(expansions)
88}pub fn completion(&self) -> &ChainCompletion
pub fn has_completion_probe(&self) -> bool
Sourcepub fn completion_marker(&self) -> Option<&Ident>
pub fn completion_marker(&self) -> Option<&Ident>
Examples found in repository?
examples/derive_field_attrs.rs (line 78)
55fn expand_field_attrs(field: &Field) -> syn::Result<Vec<TokenStream>> {
56 let field_ident = field
57 .ident
58 .as_ref()
59 .expect("named_fields only returns named struct fields");
60 let mut expansions = Vec::new();
61
62 for attr in &field.attrs {
63 if !attr.path().is_ident("attribute_dsl") {
64 continue;
65 }
66
67 let chain = attr.parse_args::<AttributeChain>()?;
68 let root = substitute_infer_in_path(chain.root_path(), &field.ty);
69
70 let calls = chain.calls().iter().map(|call| {
71 let method = call.method();
72 let turbofish = call.turbofish();
73 let args = call.args();
74 quote! { .#method #turbofish (#(#args),*) }
75 });
76
77 let completion = chain
78 .completion_marker()
79 .map(|marker| quote! { .#marker })
80 .unwrap_or_default();
81
82 expansions.push(quote! {
83 let _ = #root::builder_for(stringify!(#field_ident)) #(#calls)* #completion;
84 });
85 }
86
87 Ok(expansions)
88}pub fn span(&self) -> Span